Ext.namespace('dSS11', 'dSS11.config');

dSS11.AddOnsUpdatePanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.updateStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=addon-list-upgradable',
                    root: 'data',
                    fields: 
                    [
                        'name', 'version', 'maintainer', 
                        'description', 'homepage', 'state'
                    ]
                }
            );

            this.loaded = false;
            this.fromRefetch = false;

            var tpl = dSS11.util.defaultIPKTemplate();
            tpl.compile();

            this.mainGrid = new Ext.grid.GridPanel(
                {
                    store: this.updateStore,
                    enableColumnHide: false,
                    enableHdMenu: false,
                    columns:
                    [
                        {
                            resizable: false,
                            sortable: false,
                            hideable: false,
                            menuDisabled: true,
                            fixed: true,
                            renderer: dSS11.util.defaultAddOnIconRenderer,
                            width: 64
                        },
                        {
                            xtype: 'templatecolumn',
                            id: 'addon',
                            header: _("Apps"),
                            dataIndex: 'name',
                            sortable: true,
                            tpl: tpl
                        }
                    ],
                    trackMouseOver: true,
                    viewConfig: { forceFit: true },
                    sm: new Ext.grid.RowSelectionModel({singleSelect:true}),
                    bodyBorder: false
                }
            );

            this.noUpdatesTooltip =  new Ext.ToolTip(
                {
                    target: 'dSS11AddonsUpdateButton',
                    html: '<br/>' + 
                          _("Your apps are up to date"),
                    title: _("No updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            this.gotUpdatesTooltip = new Ext.ToolTip(
                {
                    target: 'dSS11AddonsUpdateButton',
                    html: '<br/>' + 
                          _("App updates are available for your system, click the update button to install them"),
                    title: _("Updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            Ext.apply(this,
                {
                    title: dSS11.config.gt.pgettext('verb', "Update"),
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Check for new updates"),
                            iconCls: 'icon-refresh-feed',
                            scope: this,
                            tooltip: _("Contact the server and check availability of new updates"),
                            handler: this.handleRefetch.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Update selected"),
                            ref: 'updateSelButton',
                            id: 'updateSelButton',
                            iconCls: 'icon-package-update',
                            disabled: true,
                            scope: this,
                            handler: this.updateSelectedAddon.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Update all"),
                            ref: 'dSS11AddonsUpdateButton',
                            id: 'dSS11AddonsUpdateButton',
                            iconCls: 'icon-package-update-all',
                            disabled: true,
                            scope: this,
                            handler: this.updateAddons.createDelegate(this)
                        }
                    ],
                    items: [ this.mainGrid ]
                }
            );

            dSS11.AddOnsUpdatePanel.superclass.initComponent.apply(this, arguments);

            this.on('activate',
                function() 
                {
                    if (this.loaded === false) {
                        this.updateStore.load();
                        this.loaded = true;
                    }
                },
                this);

            dSS11.MessageBus.on('refreshaddonsview', function() {
                    if (this.isVisible()) {
                        this.refreshList();
                    }
                },
                this);
            
            this.on('hide',
                function()
                {
                    this.noUpdatesTooltip.hide();
                    this.gotUpdatesTooltip.hide();
                },
                this);

            this.updateStore.on('load',
                function() 
                {
                    var button = this.getTopToolbar().get('dSS11AddonsUpdateButton');
                    var selButton = this.getTopToolbar().get('updateSelButton');
                    var count = this.updateStore.getTotalCount();

                    if (count > 0) {
                        this.noUpdatesTooltip.hide();
                        this.noUpdatesTooltip.disable();
                        this.gotUpdatesTooltip.enable();
                        button.enable();
                        this.gotUpdatesTooltip.setTitle(
                                count + " " + 
                                dSS11.config.gt.npgettext('noun', "Update",
                                                    "n% Updates", count) +
                                        " " + _("found"));
                        if (this.isVisible()) {
                            this.gotUpdatesTooltip.show();
                        } 
                    } else {
                        button.disable();
                        selButton.disable();
                        this.gotUpdatesTooltip.hide();
                        this.gotUpdatesTooltip.disable();
                        this.noUpdatesTooltip.enable();
                        if ((this.fromRefetch === true) && this.isVisible()) {
                            this.noUpdatesTooltip.show();
                        }
                    }

                    if (this.fromRefetch === true) {
                        this.findParentByType('dss11listmenupanel').getItem('dSS11AddonsFeed').refreshList();
                        this.fromRefetch = false;
                    }
                },
                this);

            this.mainGrid.getSelectionModel().on('selectionchange',
                function(sel)
                {
                    if (sel.hasSelection()) {
                        this.getTopToolbar().get('updateSelButton').enable();
                    } else {
                        this.getTopToolbar().get('updateSelButton').disable();
                    }
                },
                this);
        },
        refreshList: function() 
        {
            this.updateStore.load();
        },
        updateDone: function(jsonData, name)
        {
            var msg = _(jsonData.message);
            var title = _("Info");

            if (jsonData.code === dSS11.config.codes.ERR_OK) {
                msg = _("Apps successfully updated");
                if (name !== undefined) {
                    msg = dSS11.config.gt.strargs(
                           _("App %1 successfully updated."),
                             [ '<b>' + name + '</b>' ]);
                }
            } else if (jsonData.code === dSS11.config.codes.ERR_OPKG) {
                if (!(jsonData.opkgcode &
                    ~(dSS11.config.opkgcodes.ERR_PACKAGE_ALREADY_INSTALLED |
                      dSS11.config.opkgcodes.ERR_PACKAGE_NOT_DOWNGRADED))) {
                    // these are errors that we can usually ignore
                    msg = _("Apps successfully updated");
                    if (name !== undefined) {
                        msg = dSS11.config.gt.strargs(
                                _("App %1 successfully updated."),
                                 [ '<b>' + name + '</b>' ]);
                    }
                } else if (jsonData.opkgcode ===
                           dSS11.config.opkgcodes.ERR_DOWNLOAD_FAILED) {
                    msg = _("Failed to download packages, please try again " +
                            "later!");
                    title = _("Error");
                } else {
                    msg = _("Some apps could not be updated");
                    if (name !== undefined) {
                        msg = dSS11.config.gt.strargs(
                                _("App %1 could not be updated."),
                                 [  '<b>' + name + '</b>' ]);
                    }
                    title = _("Error");
                }
            }

            dSS11.StatusBar.setMessage(msg);
            Ext.Msg.show({
                title: title,
                msg: msg,
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshList();
            this.findParentByType('dss11listmenupanel').getItem('dSS11AddonsOverview').refreshOverview();
            this.findParentByType('dss11listmenupanel').getItem('dSS11AddonsFeed').refreshList();
        },
        refetchDone: function() 
        {
            this.fromRefetch = true;
            this.refreshList();
        },
        handleRefetch: function()
        {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'addon-update',
                              this.findParentByType('viewport').el,
                              this.refetchDone.createDelegate(this) ]);
            fn();
        },
        updateAddons: function() {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'addon-upgrade',
                              this.findParentByType('viewport').el,
                              this.updateDone.createDelegate(this) ]);
            fn();
        },
        updateSelectedAddon: function() {
            var sel = this.mainGrid.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.MessageBox.alert(_("No app selected"),
                        _("Please select an app that should be updated!"));
                return;
            }

            var record = sel.getSelected();
            var name = record.get('name');

            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'addon-upgrade',
                              this.findParentByType('viewport').el,
                              this.updateDone.createDelegate(this), name ]);
            fn();
        }

    }
);

Ext.reg('dss11addonsupdate', dSS11.AddOnsUpdatePanel);
