Ext.namespace('dSS11', 'dSS11.sys');

dSS11.sys.ApartmentProgModeForm = Ext.extend(Ext.form.FormPanel,
{
    initComponent: function()
    {
        Ext.apply(this, {
            layout: 'absolute',
            items:
            [
                {
                    xtype: 'label',
                    cls: 'x-green-text',
                    text: _("To protect the digitalStrom settings in your " +
                            "devices you can disable the activities configuration " +
                            "with the zone push button. This setting is " +
                            "valid for the whole apartment."),
                    x: 0,
                    y: 3
                },
                {
                    xtype: 'checkbox',
                    defaultType: 'checkboxfield',
                    boxLabel: _('Configuration mode enabled'),
                    name: 'pmode',
                    ref: 'programmingMode',
                    id: 'id-tls',
                    checked: true,
                    disabled: !dSS11.config.dSAIntegration,
                    x: 0,
                    y: 70
                },
                {
                    xtype: 'button',
                    text: _("Apply"),
                    minWidth: 90,
                    handler: this.setApartmentProgMode.createDelegate(this),
                    disabled: !dSS11.config.dSAIntegration,
                    x: 275,
                    y: 70
                },
                {
                    xtype: 'label',
                    cls: 'x-green-text',
                    text: '',
                    ref: 'range',
                    x: 0,
                    y: 90
                }
            ]
        });

        dSS11.sys.ApartmentProgModeForm.superclass.initComponent.apply(this, arguments);
    },
    setApartmentProgMode:  function() {
        if (dSS11.config.dSAIntegration === false) {
            return;
        }
        this.findParentByType('fieldset').ownerCt.el.mask(
            _("Applying changes") + '...', 'x-mask-loading');

        Ext.Ajax.request({
            url: '/dsa/json/hardware/setMeterProgrammingModeFlag',
            params: { mode : this.programmingMode.getValue() ? "enable" : "disable" },
            method: 'GET',
            scope: this,
            success: function(response) {
                try {
                    var jsonData = Ext.util.JSON.decode(
                                            response.responseText);

                    if (jsonData.success !== true) {
                        Ext.Msg.alert(_("Error"), jsonData.message);
                        this.findParentByType('fieldset').ownerCt.el.unmask();
                        this.reloadApartmentProgMode();
                        return;
                    }
                    dSS11.StatusBar.setMessage(
                        _("Successfully set mode"));
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                        _("Could not store setting") +
                            ": "  + error.message);
                }

                this.findParentByType('fieldset').ownerCt.el.unmask();
                this.reloadApartmentProgMode();
            },
            failure: function(response) {
                var errMsg = "";
                try {
                    var json = Ext.util.JSON.decode(response.responseText);
                    if (json.message) {
                        errMsg = ": " + json.message;
                    }
                } catch (err) {}
                Ext.Msg.alert(_("Error"),
                              _("Could not store setting") +
                                errMsg);
                this.findParentByType('fieldset').ownerCt.el.unmask();
                this.reloadApartmentProgMode();
            }
        });
    },
    reloadApartmentProgMode: function() {
        if (dSS11.config.dSAIntegration === false) {
            return;
        }
        this.findParentByType('fieldset').ownerCt.el.mask(
            _("Loading settings") + "...", 'x-mask-loading');

        this.range.setText('');
        Ext.Ajax.request({
            url: '/dsa/json/hardware/getMeterProgrammingModeFlag',
            method: 'GET',
            scope: this,
            disableCaching: true,
            success: function(response) {
                try {
                    if (response.responseText.length === 0) {
                        // dSA is not running...
                        return;
                    }
                    var jsonData = Ext.util.JSON.decode(
                                            response.responseText);

                    if (jsonData.success !== true) {
                        Ext.Msg.alert(_("Error"), jsonData.message);
                        this.findParentByType('fieldset').ownerCt.el.unmask();
                        return;
                    }

                    this.programmingMode.setValue(jsonData.data.progmode);

                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                        _("Could not load settings") +
                            ": "  + error.message);
                }

                this.findParentByType('fieldset').ownerCt.el.unmask();
            },
            failure: function(response) {
                this.range.setText('');
                var errMsg = "";
                try {
                    var json = Ext.util.JSON.decode(response.responseText);
                    if (json.message) {
                        errMsg = ": " + json.message;
                    }
                } catch (err) {}
                Ext.Msg.alert(_("Error"),
                              _("Could not load settings") +
                                errMsg);
                this.findParentByType('fieldset').ownerCt.el.unmask();
            }
        });
    }
});

Ext.reg('dss11apartmentprogmodeform', dSS11.sys.ApartmentProgModeForm);
