Ext.namespace('dSS11', 'dSS11.sys');

dSS11.sys.HostNameForm = Ext.extend(Ext.form.FormPanel,
{
    initComponent: function()
    { 
        Ext.apply(this,
        {
            height: 70,
            layout: 'absolute',
            items:
            [
                {
                    xtype: 'label',
                    cls: 'x-green-text',
                    text: _("Note: the name must not contain any whitespaces" +
                            " and special characters (except '-')"),
                    x: 0,
                    y: 0,
                    height: 30
                },
                {
                    xtype: 'label',
                    text: _("Name") + ':',
                    x: 0,
                    y: 43
                },
                {
                    xtype: 'textfield',
                    width: 160,
                    name: 'hostname',
                    allowBlank: false,
                    maxLength: 64,
                    validator: this.validateHostName,
                    x: 100,
                    y: 40
                },
                {
                    xtype: 'button',
                    text: _("Apply"),
                    minWidth: 90,
                    handler: this.setSystemName.createDelegate(this),
                    x: 275,
                    y: 40
                }
            ]
        });

        dSS11.sys.HostNameForm.superclass.initComponent.apply(this, arguments);
    },
    setSystemName:  function() {
        if (!this.getForm().isValid()) {
            Ext.Msg.alert(_("Error"), _("The name you entered is invalid! Please do not use any whitespaces or special characters."));
            return;
        }
        var maskEl = this.findParentByType('fieldset').ownerCt.el;
        maskEl.mask(_("Applying changes") + '...', 'x-mask-loading');

        var name = this.getForm().getValues().hostname;

        // fire and forget: the dSS name is not used anywhere anyway
        dSS11.util.setApartmentName(name);

        this.getForm().submit({
            url: '/cgi-bin/setnamewrapper.sh',
            method: 'GET',
            scope: this,
            success: function(form, action) {
                maskEl.unmask();
                try {
                    var jsonData = Ext.util.JSON.decode(
                                                action.response.responseText);
                    if ((jsonData.code === dSS11.config.codes.ERR_REDIRECT) &&
                        (jsonData.ip.length > 0)) {
                        this.findParentByType('viewport').el.mask(
                            _("IP address has changed, redirecting to " +
                              "new location at") +
                              " http://" + jsonData.ip + "/",
                              'x-mask-loading');
                        window.location = 'http://' + jsonData.ip;
                    } else if ((jsonData.code ===
                                dSS11.config.codes.ERR_REDIRECT_NAME) &&
                              (jsonData.ip.length > 0)) {
                        if (dSS11.util.validateIP(window.location.host) ===
                            false) {
                            this.findParentByType('viewport').el.mask(
                                _("Redirecting to new location") + "...",
                                'x-mask-loading');
                            window.location = 'http://' + jsonData.ip;
                        }
                    } else if (jsonData.code !== dSS11.config.codes.ERR_OK) {
                        Ext.Msg.alert(_("Error"), (jsonData.message));
                        this.reloadHostName();
                    }
                    dSS11.StatusBar.setMessage(_("Name set successfully"));
                } catch (error) {
                    Ext.Msg.alert(_("Error"), error);
                }
            },
            failure: function(form, action) {
                maskEl.unmask();
                switch (action.failureType) {
                    case Ext.form.Action.CONNECT_FAILURE:
                        Ext.Msg.alert(_("Error"),
                            _("Ajax communication failed"));
                    break;
                    default:
                        Ext.Msg.alert(_("Error"),
                                _("Could not save installation name"));
                    break;
                }
            }
        });
    },
    validateHostName: function(name) {
        if ((name.length === 0) || (name.length > 64)) {
            return false;
        }

        var expr = /[^0-9a-zA-Z\-]/;
        if (expr.exec(name) === null) {
            return true;
        }

        return false;
    },
    reloadHostName: function() {
        this.findParentByType('fieldset').ownerCt.el.mask(_("Loading system name") + "...",
                  'x-mask-loading');
                    
        this.load({
            url: '/cgi-bin/getname.sh',
            method: 'GET',
            scope: this,
            disableCaching: true,
            success: function(form, action) {
                try {
                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);

                    if (jsonData.success !== true) {
                        Ext.Msg.alert(_("Error"), jsonData.message);
                        this.findParentByType('fieldset').ownerCt.el.unmask();
                        return;
                    }
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                        _("Could not load installation name") +
                            ": "  + error.message);
                }

                this.findParentByType('fieldset').ownerCt.el.unmask();
            },
            failure: function(form, action) {
                switch (action.failureType) {
                    case Ext.form.Action.CONNECT_FAILURE:
                        Ext.Msg.alert(_("Error"),
                            _("Ajax communication failed"));
                    break;
                    default:
                        Ext.Msg.alert(_("Error"),
                                _("Could not load installation name"));
                    break;
                }

                this.findParentByType('fieldset').ownerCt.el.unmask();
            }
        });
    }
});

Ext.reg('dss11syshostnameform', dSS11.sys.HostNameForm);
