Ext.namespace('dSS11', 'dSS11.sys');

dSS11.sys.LocationForm = Ext.extend(Ext.form.FormPanel,
{
    initComponent: function()
    {
        // Schlieren
        this.selectedCoordinates = {
            latitude: 47.401632,
            longitude: 8.44314,
            timezone: null,
            gmtoffset: 1.0
        };

        this.lastCoordinates = {};

        this.countryStore = new Ext.data.JsonStore({
            url: dSS11.config.langPath + 'countries.json',
            root: 'data.countries',
            fields: [ 'id', 'name' ],
            sortInfo: { field: 'name', direction: 'ASC' }
        });

        this.cityStore = new Ext.data.JsonStore({
            url: '/geodata/geodata_CH.json',
            autoLoad: false,
            root: 'data.cities',
            fields:
            [
                'country',
                'city',
                'latitude',
                'longitude',
                'timezone',
                'gmtoffset'
            ],
            sortInfo: { field: 'city', direction: 'ASC' }
        });

        this.defaultTZHint = _("The time zone under 'Date and time' " +
                             "will automatically be set depending " +
                             "on the locality selected in the " +
                             "list.");
        this.altTZHint = _("The time zone under 'Data and time' must be " +
                           "set manually if no locality was selected from " +
                           "the list.");

        var locSet =
            {
                xtype: 'fieldset',
                title: _("Location"),
                ref: 'fset',
                layout: 'form',
                items: [
                    {
                        xtype: 'panel',
                        scope: this,
                        height: 185,
                        layout: 'absolute',
                        bodyBorder: false,
                        ref: 'comboPanel',
                        items: [
                            {
                                xtype: 'label',
                                text: _("Country") + ': ',
                                x: 0,
                                y: 3
                            },
                            {
                                xtype: 'combo',
                                lazyInit: false,
                                mode: 'local',
                                name: 'countries',
                                ref: 'countries',
                                forceSelection: true,
                                store: this.countryStore,
                                emptyText: _("Select country") + "...",
                                triggerAction: 'all',
                                typeAhead: true,
                                displayField: 'name',
                                valueField: 'id',
                                width: 220,
                                x: 146,
                                y: 0,
                                listeners: {
                                    select: this.loadCities.createDelegate(this)
                                }
                            },
                            {
                                xtype: 'label',
                                text: _("Locality") + ': ',
                                x: 0,
                                y: 35
                            },
                            {
                                xtype: 'combo',
                                lazyInit: true,
                                mode: 'local',
                                name: 'cities',
                                ref: 'cities',
                                disabled: true,
                                forceSelection: true,
                                store: this.cityStore,
                                emptyText: _("Select locality") + "...",
                                triggerAction: 'all',
                                typeAhead: true,
                                displayField: 'city',
                                listeners: {
                                    select: this.prepareCoordinates.createDelegate(
                                                                        this)
                                },
                                width: 220,
                                x: 146,
                                y: 35
                            },
                            {
                                xtype: 'label',
                                text: _("Coordinates") + ': ',
                                x: 0,
                                y: 70
                            },
                            {
                                xtype: 'label',
                                x: 146,
                                y: 70,
                                ref: 'coordinates'
                            },
                            {
                                xtype: 'label',
                                text: _("Sunrise/Sunset") + ': ',
                                x: 0,
                                y: 105
                            },
                            {
                                xtype: 'label',
                                ref: 'solar',
                                x: 146,
                                y: 105
                            },
                            {
                                xtype: 'label',
                                ref: 'tzhint',
                                text: this.defaultTZHint,
                                x: 0,
                                y: 140
                            }
                        ]
                    }, // country panel
                    {
                        xtype: 'box',
                        height: 10
                    },
                    {
                        bodyBorder: false,
                        xtype: 'panel',
                        layout: 'absolute',
                        ref: 'bpanel',
                        height: 25,
                        items: [
                            {
                                xtype: 'button',
                                text: _("Select location on map"),
                                disabled: true,
                                hidden: true,
                                ref: 'mapbutton',
                                minWidth: 190,
                                x: 0,
                                y: 0,
                                handler: this.showMap.createDelegate(this)
                            },
                            {
                                xtype: 'button',
                                text: _("Apply"),
                                ref: 'applybutton',
                                disabled: true,
                                minWidth: 90,
                                x: 275,
                                y: 0,
                                handler: this.applyLocationSettings.createDelegate(this)
                            }
                        ]
                    }
                ] // timeset items
            };

        Ext.apply(this, {
            padding: 6,
            width: 400,
            items: [ locSet ]
        });

        dSS11.sys.LocationForm.superclass.initComponent.apply(this, arguments);

        try {
            this.suncalc = new SunCalc();
        } catch (err) {
            this.suncalc = {};
            this.suncalc.compute_current_date = function() {
                return { "sunrise": "", "sunset": ""};
            };
        }

        // we entered the UI while an update was in progress,
        // the UI will be force-reloaded after the update, so do not try to
        // load any data here
        if (dSS11.config.updateInProgress !== true) {
            this.countryStore.load();
        }

        this.cityStore.on('load', function() {
            this.fset.comboPanel.cities.clearValue();
            this.fset.comboPanel.cities.enable();
            this.fset.bpanel.mapbutton.disable();
            this.fset.bpanel.applybutton.disable();
        },
        this);

        this.countryStore.on('load', function(store, records, options) {
            var rec = new store.recordType({
                id: '-',
                name: _("other country")
            });
            store.insert(0, rec);
        },
        this);

        this.on('afterrender', function() {
            this.maskEl = this.findParentByType('viewport').getEl();

            if (this.mapScript === undefined) {
                Ext.Ajax.request({
                    url: '/cgi-bin/checknet.sh',
                    method: 'GET',
                    scope: this,
                    success: function(response) {
                        try {
                            var jsonData = Ext.util.JSON.decode(
                                                        response.responseText);
                            if (jsonData.success === true) {
                                this.mapScript =
                                    document.createElement("script");
                                this.mapScript.type = "text/javascript";
                                this.mapScript.src =
                                    'https://maps.googleapis.com/maps/api/js' +
                                    '?v=3.5&sensor=false&' +
                                    'callback=GoogleMapsCallback';
                                document.body.appendChild(this.mapScript);
                            }
                        } catch (err) { }
                    },
                    failure: function(response) {}
                });
            }
        },
        this);
        dSS11.MessageBus.on('googlemapsloaded', function() {
            this.fset.bpanel.mapbutton.setVisible(true);
        },
        this);
    },
    reloadLocationSettings: function() {
        this.el.mask(_("Loading location settings") + "...",
                         'x-mask-loading');
        Ext.Ajax.request({
            url: '/cgi-bin/readloc.sh',
            method: 'GET',
            scope: this,
            success: function(response) {
                this.el.unmask();
                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.success !== true) {
                        dSS11.StatusBar.setMessage(_(jsonData.message));
                        return;
                    }
                    var country = jsonData.data.country;
                    var city = dSS11.util.decodeHTMLentity(
                                                    jsonData.data['location']);
                    this.fset.comboPanel.countries.setValue(country);
                    this.selectedCoordinates.gmtoffset =
                                                    jsonData.data.gmtoffset;
                    this.selectedCoordinates.latitude = jsonData.data.latitude;
                    this.selectedCoordinates.longitude =
                                                        jsonData.data.longitude;

                    this.fset.comboPanel.cities.disable();

                    if (country === '-') { // other
                        this.fset.bpanel.mapbutton.enable();
                        this.fset.comboPanel.cities.clearValue();
                        this.fset.comboPanel.tzhint.update(this.altTZHint);
                        this.selectedCoordinates.timezone = null;
                        this.showCoordinates(jsonData.data.latitude,
                                             jsonData.data.longitude,
                                             null);
                    } else {
                        this.fset.comboPanel.tzhint.update(this.defaultTZHint);

                        if (this.citiesLoaded !== country) {
                            this.cityStore.clearFilter();
                            this.cityStore.proxy.setUrl('/geodata/geodata_' +
                                                        country + '.json');
                            this.cityStore.reload({ callback: function() {
                                    this.fset.comboPanel.cities.setValue(city);
                                    this.showCoordinates(jsonData.data.latitude,
                                                     jsonData.data.longitude,
                                                     null);
                                    this.fset.bpanel.mapbutton.enable();
                                }.createDelegate(this)
                            });
                            this.citiesLoaded = country;
                        } else {
                            this.fset.comboPanel.cities.setValue(city);
                            this.showCoordinates(jsonData.data.latitude,
                                                 jsonData.data.longitude,
                                                 null);

                            this.fset.comboPanel.cities.enable();
                            this.fset.bpanel.mapbutton.enable();
                        }
                    }
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                                  _("Could not load location settings") +
                                    ": " + error);
                }
            },
            failure: function(response) {
                var msg = _("Could not load location settings");
                try {
                    dSS11.util.checkResponse(response);
                } catch (error) {
                    msg = msg + ": " + _(error);
                }

                Ext.Msg.alert(_("Error"), msg);
            }
        });
    },
    applyLocationSettings:  function() {
        this.maskEl.mask(_("Applying changes") + '...', 'x-mask-loading');

        Ext.Ajax.request({
            url: '/cgi-bin/locationwrapper.sh',
            method: 'GET',
            scope: this,
            params: {
                country:  this.fset.comboPanel.countries.getValue(),
                "location": this.fset.comboPanel.cities.getValue(),
                latitude: this.selectedCoordinates.latitude,
                longitude: this.selectedCoordinates.longitude,
                gmtoffset: this.selectedCoordinates.gmtoffset,
                timezone: this.selectedCoordinates.timezone
            },
            success: function(response) {
                this.maskEl.unmask();
                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.success !== true) {
                        Ext.Msg.alert(_("Error"), _(jsonData.message));
                        return;
                    }

                    dSS11.StatusBar.setMessage(_("Location settings applied " +
                                                 "successfully."));

                    dSS11.MessageBus.fireEvent('refreshdatetime');
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                                  _("Could not apply location settings") +
                                    ": " + error);
                }
            },
            failure: function() {
                this.maskEl.unmask();
                Ext.Msg.alert(_("Error"),
                              _("Could not apply location settings"));
            }
        });
    },
    loadCities: function(combo, record, index) {
        var country = record.get('id');
        this.fset.comboPanel.coordinates.update("&nbsp;");
        this.fset.comboPanel.solar.update("&nbsp;");
        this.fset.bpanel.applybutton.disable();

        // we can only rely on the map...
        if (country === '-') {
            this.fset.bpanel.mapbutton.enable();
            this.fset.comboPanel.cities.clearValue();
            this.fset.comboPanel.cities.disable();
            this.selectedCoordinates.timezone = null;
            this.selectedCoordinates.gmtoffset = null;
            this.fset.comboPanel.tzhint.update(this.altTZHint);
            return;
        } else {
            this.fset.comboPanel.tzhint.update(this.defaultTZHint);
        }

        if (this.citiesLoaded !== country) {
            this.cityStore.clearFilter();
            this.fset.bpanel.mapbutton.disable();
            this.cityStore.proxy.setUrl('/geodata/geodata_' + country +
                                        '.json');
            this.cityStore.reload();
            this.citiesLoaded = country;
        } else {
            this.fset.comboPanel.cities.clearValue();
            this.fset.comboPanel.cities.enable();
        }
    },
    prepareCoordinates: function(combo, record, index) {
        var lat = record.get('latitude');
        var lng = record.get('longitude');
        var timezone = record.get('timezone');
        var gmtoffset = record.get('gmtoffset');

        this.selectedCoordinates = {
            latitude: lat,
            longitude: lng,
            timezone: timezone,
            gmtoffset: gmtoffset
        };

        this.showCoordinates(lat, lng, timezone);
        this.fset.bpanel.mapbutton.enable();
    },
    showCoordinates: function(latitude, longitude, timezone) {
        var c = String.format("{0}&deg; N / {1}&deg; W",
                              dSS11.util.toFixed(latitude, 5),
                              dSS11.util.toFixed(longitude, 5));
        this.fset.comboPanel.coordinates.update(c);
        this.fset.bpanel.applybutton.enable();

        longitude = -longitude;

        Ext.Ajax.request({
            url: '/cgi-bin/dateinfowrapper.sh',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: { nozones: true },
            success: function(response) {
                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.success !== true) {
                        Ext.Msg.alert(_("Error"), _(jsonData.message));
                        return;
                    }

                    var date = jsonData.data.date.split('-');

                    var zoneOffset = jsonData.data.gmtoffset;
                    if (this.selectedCoordinates.gmtoffset === null) {
                        this.selectedCoordinates.gmtoffset = zoneOffset;
                    }

                    if (this.selectedCoordinates.timezone === null) {
                        this.selectedCoordinates.timezone =
                                                        jsonData.data.timezone;
                    }

                    var solar = this.suncalc.compute_current_date(
                                latitude,
                                longitude,
                                parseInt(dSS11.util.rmLeadingZeroes(date[2]),
                                         10),
                                parseInt(dSS11.util.rmLeadingZeroes(date[1]),
                                         10),
                                parseInt(dSS11.util.rmLeadingZeroes(date[0]),
                                         10),
                                zoneOffset);

                    if ((solar.sunrise == "[Above]") ||
                        (solar.sunset == "[Above]") ||
                        (solar.sunrise == "[Below]") ||
                        (solar.sunset == "[Below]")) {
                        this.fset.comboPanel.solar.update("&nbsp;");
                    } else {
                        this.fset.comboPanel.solar.update(
                            String.format("{0} / {1}  " + jsonData.data.zoneshort,
                                          solar.sunrise, solar.sunset));
                    }
                } catch (error) {
                     Ext.Msg.alert(_("Error"),
                             _("Could not determine sunrise and sunset times") +
                             ": " + _(error));
                }
            },
            failure: function(response) {
                Ext.Msg.alert(_("Error"),
                            _("Could not determine sunrise and sunset times"));
            }
        });
    },
    showMap: function() {
        if (!this.mapwin) {
            Ext.copyTo(this.lastCoordinates, this.selectedCoordinates,
                       'latitude,longitude,timezone,gmtoffset');

            var addr = String.format("{0},{1}",
                                     this.selectedCoordinates.latitude,
                                     this.selectedCoordinates.longitude);

            this.mapPanel = new Ext.ux.GMapPanel({
                zoomLevel: 14,
                minZoom: 8,
                maxZoom: 20,
                gmapType: 'map',
                region: 'center',
                scope: this,
                respErrorCallback: function(code, msg) {
                    Ext.Msg.alert(_("Error"), _(msg));
                },
                minGeoAccuracy: 'APPROXIMATE',
                disabled: true,
                mapConfOpts:
                [
                    'enableScrollWheelZoom',
                    'enableDoubleClickZoom',
                    'enableDragging'
                ],
                mapControls:
                [
                    'GMapTypeControl',
                    'NonExistantControl'
                ],
                setCenter: {
                    geoCodeAddr: addr,
                    marker: {
                        draggable: true
                    },
                    listeners: {
                        dragend: this.markerDragged.createDelegate(this)
                    }
                },
                listeners: {
                    mapready: this.mapReady.createDelegate(this)
                }
            });

            this.search = new Ext.form.TextField({
                minWidth: 200,
                flex: 1
            });

            this.search.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    this.addrSearch();
                }
            },
            this);

            this.searchButton = {
                xtype: 'button',
                text: _("Search"),
                minWidth: 100,
                scope: this,
                handler: this.addrSearch.createDelegate(this)
            };

            this.centerButton = {
                xtype: 'button',
                text: _("Center marker on map"),
                scope: this,
                handler: this.centerOnMap.createDelegate(this)
            };

            this.mapwin = new Ext.Window({
                layout: 'border',
                title: _("Select your location"),
                closeAction: 'hide',
                width: 750,
                height: 500,
                scope: this,
                modal: true,
                x: 40,
                y: 60,
                items:
                [
                    {
                        xtype: 'panel',
                        height: 35,
                        region: 'north',
                        hideBorders: true,
                        padding: 6,
                        layout: 'hbox',
                        items:
                        [
                            this.search,
                            {
                                xtype: 'box',
                                width: 10
                            },
                            this.searchButton
                        ]
                    },
                    {
                        xtype: 'panel',
                        height: 35,
                        region: 'south',
                        layout: 'hbox',
                        padding: 6,
                        hideBorders: true,
                        bodyBorder: false,
                        scope: this,
                        items:
                        [
                            this.centerButton,
                            {
                                xtype: 'box',
                                flex: 4
                            },
                            {
                                xtype: 'button',
                                text: _("Apply"),
                                minWidth: 100,
                                scope: this,
                                handler: function() {
                                    this.centerOnMap();
                                    if (this.mapwin) {
                                        this.mapwin.hide();
                                    }
                                    this.acceptGeoPosition();
                                }
                            },
                            {
                                xtype: 'box',
                                width: 10
                            },
                            {
                                xtype: 'button',
                                text: _("Cancel"),
                                scope: this,
                                minWidth: 100,
                                handler: function() {
                                    Ext.copyTo(this.selectedCoordinates,
                                               this.lastCoordinates,
                                       'latitude,longitude,timezone,gmtoffset');

                                    this.centerOnMap();
                                    if (this.mapwin) {
                                        this.mapwin.hide();
                                    }
                                }
                            }
                        ]
                    },
                    this.mapPanel
                ]
            });

        } else {
            if (this.compareCoordinates(this.lastCoordinates,
                                   this.selectedCoordinates) === false) {
                this.mapPanel.geoCodeLookup(String.format("{0},{1}",
                                            this.selectedCoordinates.latitude,
                                            this.selectedCoordinates.longitude),
                                            {
                                                draggable: true
                                            },
                                            true,
                                            true,
                                            {
                                                dragend: this.markerDragged.createDelegate(this)
                                            });

                Ext.copyTo(this.lastCoordinates, this.selectedCoordinates,
                        'latitude,longitude,timezone,gmtoffset');
            }
        }

        this.mapwin.show();
    },
    compareCoordinates: function(a, b) {
        return ((a.latitude === b.latitude) && (a.longitude === b.longitude) &&
                (a.timezone === b.timezone) && (a.gmtoffset === b.gmtoffset));
    },
    markerDragged: function(evt) {
        this.selectedCoordinates.latitude = evt.latLng.Na;
        this.selectedCoordinates.longitude = evt.latLng.Oa;
    },
    markerAdded: function(marker, point) {
        this.selectedCoordinates.latitude = point.Na;
        this.selectedCoordinates.longitude = point.Oa;
    },
    addrSearch: function() {
        var searchString = this.search.getValue();
        if (searchString.length > 0) {
            this.search.setValue("");
            this.mapPanel.geoCodeLookup(searchString,
                                        {
                                            draggable: true,
                                            callback: this.markerAdded.createDelegate(this)
                                        },
                                        true,
                                        true,
                                        {
                                            dragend: this.markerDragged.createDelegate(this)
                                        });
        }
    },
    centerOnMap: function() {
        var point = new google.maps.LatLng(this.selectedCoordinates.latitude,
                                           this.selectedCoordinates.longitude);
        this.mapPanel.addMarker(
            point,
            {
                draggable: true
            },
            true,
            true,
            {
                dragend: this.markerDragged.createDelegate(this)
            }
        );
    },
    acceptGeoPosition: function() {
        this.showCoordinates(this.selectedCoordinates.latitude,
                             this.selectedCoordinates.longitude,
                             this.selectedCoordinates.timezone);
    },
    mapReady: function(panel, map) {
         panel.enable();
    }
});

Ext.reg('dss11syslocationform', dSS11.sys.LocationForm);
